#ifndef __IMMS_H
#define __IMMS_H

#include <string>
#include <fstream>
#include <list>
#include <memory>

#include "sqldb.h"
#include "songinfo.h"

#define ROUND(x) (int)((x) + 0.5)

#define FIRST_SKIP_RATE -8
#define CONS_SKIP_RATE -4
#define FIRST_NON_SKIP_RATE 6
#define JUMPED_FROM -1
#define JUMPED_TO_SKIPPED 1
#define JUMPED_TO_FINNISHED 8
#define CONS_NON_SKIP_RATE 2

#define QUALITY_BIAS 0.1
#define THRESHOLD_GROW 0.35
#define THRESHOLD_DECAY 0.1
#define EXPONENT_STEP 3
#define TIMEHOLD_STEP 0.25
#define MAX_TIME 7*24*60
#define BASE_BIAS 1

#define TERM_WIDTH 80

using std::string;
using std::ofstream;
using std::endl;
using std::auto_ptr;

// IMMS, UMMS, we all MMS for XMMS?

class Imms
{
public:
    Imms();

    // Public interface

    bool check_song(const string &path);
    void start_song(const string &path);
    void end_song(bool at_the_end, bool jumped, bool bad);

    void setup(const char* _email, int _quality, bool _quality);

protected:
    // Helper functions

    void update_thresholds(const string &path);
    void fetch_song_info(const string &path);
    void parse_song_info(const string &path);
    void print_song_info(const string &path);

    // State variables

    string email;
    int quality;
    double threshold;
    int timehold;
    bool last_skipped, last_jumped, did_check, verbose;

    // Helper objects

    auto_ptr<SqlDb> sqldb;
    ofstream fout;
    SongInfo songinfo;

    // Song info cache

    int rating, last_timehold;
    bool identified, unrated;
    time_t last_time;
    double last_threshold;
    string artist, title;
};

#endif
