#ifndef __STRMANIP_H
#define __STRMANIP_H

#include <string>
#include <sstream>
#include <list>
#include <set>
#include <memory>
#include <ctype.h>

using namespace std;

inline string itos(unsigned int i)
{
    ostringstream str;
    str << i;
    return str.str();
};

// Double up single quotes to escape them sqlite style
inline string escape_string(const string &es)
{
    string s = es;
    unsigned int pos = 0;
    while (1)
    {
        pos = s.find('\'', pos);
        if (pos == string::npos)
            break;
        s.insert(pos, 1, '\'');
        pos += 2;
    }
    return s;
}

inline string strtime(double mins)
{
    int hours = (int)mins / 60;
    if (!hours)
        return "0h";

    ostringstream s;
    if (hours > 23)
    {
        if (hours / 24)
            s << hours / 24 << "d";
        hours %= 24;
    }
    if (hours)
        s << hours << "h";
    return s.str();
};               

inline string string_tolower(string s)
{
    for (string::iterator i = s.begin(); i != s.end(); i++)
       *i = tolower(*i);

    return s;
};

inline string string_brfilter(string s)
{
    int brackets = 0;
    string news = "";
    for (string::iterator i = s.begin(); i != s.end(); i++)
    {
        char c = *i;
        if (c == '[' || c == '{' || c == '(')
        {
            brackets++;
            // Replace brackets with a "/" to act as a separator
            news += "/";
        }
        if (!brackets)
        {
            news += *i;
        }
        if ((c == ']' || c == '}' || c == ')') && brackets > 0)
        {
            brackets--;
        }
    }
    return news;
};            

inline string string_replace(string s, char c, char r)
{
    for (string::iterator i = s.begin(); i != s.end(); i++)
    {
        if (*i == c)
            *i = r;
    }
    return s;
};            

inline string string_normalize(string s)
{
    s = string_brfilter(string_tolower(string_replace(s, '_', ' ')));
    string news = "";
    for (string::iterator i = s.begin(); i != s.end(); i++)
    {
        if (islower(*i))
            news += *i;
    }
    return news;
};            

void string_chr_split(set<string> &store, const string &s,
        string delim = "\t\r\n ", string anti = "\"");

list<string> &delimlst();

void string_str_split(set<string> &store, const string &s,
        const list<string> &delims = delimlst());

inline int string_count(const string &s, char c)
{
    int count = 0;
    for (string::const_iterator i = s.begin(); i != s.end(); i++)
    {
        count += (*i == c);
    }

    return count;
};

void imms_magic_parse(set<string> &store, string s);

typedef char lev_byte;

size_t
lev_edit_distance(size_t len1, const lev_byte *string1,
                  size_t len2, const lev_byte *string2,
                  size_t xcost = 0);

inline bool string_like(const string &s1, const string &s2)
{
    int len1 = s1.length();
    int len2 = s2.length();

    int distance = lev_edit_distance(len1, s1.c_str(), len2, s2.c_str());
    return (len1 + len2) / 8 >= distance;
}

#endif
